/*
加密解密算法。
从之前实时识别的私有化部署里的接入层的代码里抽取出来。

*/
#ifndef _AES_ENCRYPT_H_
#define _AES_ENCRYPT_H_

#include <string>

using namespace std;

#define SAFE_FREE(p) {if(p) free(p); p=NULL;}




/*
md5加密
params:
	str: 需要加密的字符串 
return:
	加密之后的字符串
*/
std::string str_md5(const std::string &str);






/*
sha1加密字符串
params:
	str: 需要加密的字符串 
return:
	加密之后的字符串
*/
std::string str_sha1(const std::string &str);






/*
sha1加密文件
params:
	filePath: 需要加密的文件路径
return:
	加密之后的字符串
*/
std::string file_sha1(const std::string &filePath);





/*
base64_encode
params:
	bytes_to_encode: 输入数据指针
	in_len： 输入数据长度
return:
	base64_encode之后的字符串
*/
std::string base64_encode(unsigned char const* bytes_to_encode, unsigned int in_len);






/*
base64_decode
需要保证存放输出数据的有足够的空间
params:
	base64_str: 输入数据
	bindata： 保存输出的指针
return:
	输出数据的长度
*/
int base64_decode(const std::string& base64_str, unsigned char *bindata);






/*
sha1mac_encode
params:
	key: 加密的key
	key_length： key的长度
	input： 输入数据
	input_length: 输入数据长度
	output： 存放输出的指针
	output_length: 输出数据的长度
return:
	总是返回0
*/
int sha1mac_encode(const char * key, unsigned int key_length,  
                const char * input, unsigned int input_length,  
                unsigned char * output, unsigned int &output_length); 
		





string sha256(const string str);

string string2hex(char* data, int len);

void str2low(string& str);

int sha256mac_encode(const char * key, unsigned int key_length,  
                const char * input, unsigned int input_length,  
                unsigned char * output, unsigned int &output_length) ;

int sha256mac(const char* key, int key_len, const char* sign, int sign_len ,unsigned char* out, unsigned int &out_len);

		
/*
urlEncode
params:
	str: 输入数据
return:
	输出数据
*/
std::string urlEncode(const std::string& str);






/* 
aes加密
params:
	input_str: 需要加密的字符串 
	key_str: 加密的key
	output_str: 加密结果，通过引用返回
return:
	0: 正常
	-1：输入字符串或者key为空
	-2：设置加密key错误
	-3：分配内存失败
*/ 
int aes_encrypt(const string &input_str, const char* key_str, string &output_str);






/* aes解密
params:
	input_str: 需要解密的字符串 
	key_str: 解密的key
	output_str: 解密结果，通过引用返回
return:
	0: 正常
	-1：输入字符串或者key为空
	-2：设置加密key错误
	-3：分配内存失败
*/ 
int aes_decrypt(const string &input_str, const char* key_str, string &output_str);

#endif
