#include <openssl/aes.h>
#include "openssl/md5.h"
#include "openssl/hmac.h"
#include "openssl/sha.h"
#include <fstream>
#include <string.h>
#include <iomanip>
#include <sstream>
#include <iostream>

#include "encrypt.h"

#define READ_BUF_SIZE (2*1024*1024)

string str_md5(const string &str)
{
    MD5_CTX ctx;
    unsigned char md5[MD5_DIGEST_LENGTH]={0};

    MD5_Init(&ctx);
    MD5_Update(&ctx, str.c_str(), str.size());
    MD5_Final(md5,&ctx);

    string md5_str;
    char tmp[3];
    for(int i = 0; i < MD5_DIGEST_LENGTH; ++i)
    {
        snprintf(tmp, 3, "%02x", md5[i]);
        md5_str += tmp;
    }
    return md5_str;
}

string str_sha1(const string &str)
{
    static char hexBytes[16] = {
    '0', '1', '2', '3', '4', '5', '6', '7', '8',
    '9', 'a', 'b', 'c', 'd', 'e', 'f'};

    unsigned char sha1[SHA_DIGEST_LENGTH];
    
    SHA_CTX ctx;
    SHA1_Init(&ctx);
    SHA1_Update(&ctx, str.c_str(), str.size());
    SHA1_Final(sha1, &ctx);
    string shaStr = "";
    for (int i=0; i<SHA_DIGEST_LENGTH; i++) {
        shaStr += hexBytes[sha1[i] >> 4];
        shaStr += hexBytes[sha1[i] & 0x0f];
    }
    return shaStr;
}

string sha256(const string str)
{
    unsigned char hash[SHA256_DIGEST_LENGTH];
    SHA256_CTX sha256;
    SHA256_Init(&sha256);
    SHA256_Update(&sha256, str.c_str(), str.size());
    SHA256_Final(hash, &sha256);
    stringstream ss;
    for(int i = 0; i < SHA256_DIGEST_LENGTH; i++)
    {
        ss << hex << setw(2) << setfill('0') << (int)hash[i];
    }
    return ss.str();
}

string string2hex(char* data, int len)
{
    const std::string hex = "0123456789ABCDEF";
    std::stringstream ss;

    for (int i = 0; i < len; ++i)
        ss << hex[(unsigned char)data[i] >> 4] << hex[(unsigned char)data[i] & 0xf];
    return ss.str();
}

void str2low(string& str){
    for (int i = 0; i < str.size(); i++)
    {
        str[i] = tolower(str[i]);
    }
}

string file_sha1(const string &filePath) {

    static char hexBytes[SHA_DIGEST_LENGTH] = {
    '0', '1', '2', '3', '4', '5', '6', '7', '8',
    '9', 'a', 'b', 'c', 'd', 'e', 'f'};

    unsigned char sha1[SHA_DIGEST_LENGTH];
    ifstream fileInput(filePath.c_str(),
            ios::in | ios::binary);
    if (!fileInput) {
        return "";
    }

    char buf[READ_BUF_SIZE];
    SHA_CTX ctx;
    SHA1_Init(&ctx);

    uint64_t len = 0;
    while(!fileInput.eof()) {
        fileInput.read(buf, READ_BUF_SIZE);
        len = fileInput.gcount();
        SHA1_Update(&ctx, buf, len);
    }

    SHA1_Final(sha1, &ctx);

    fileInput.close();

    string shaStr = "";
    for (int i = 0; i < SHA_DIGEST_LENGTH; i++) {
        shaStr += hexBytes[sha1[i] >> 4];
        shaStr += hexBytes[sha1[i] & 0x0f];
    }

    return shaStr;
}

int sha1mac_encode(const char * key, unsigned int key_length,  
                const char * input, unsigned int input_length,  
                unsigned char * output, unsigned int &output_length) {  
    const EVP_MD * engine = EVP_sha1();  
    HMAC_CTX ctx;  
    HMAC_CTX_init(&ctx);  
    HMAC_Init_ex(&ctx, key, key_length, engine, NULL);  
    HMAC_Update(&ctx, (unsigned char*)input, input_length);        // input is OK; &input is WRONG !!!  
  
    HMAC_Final(&ctx, output, &output_length);  
    HMAC_CTX_cleanup(&ctx);  
  
    return 0;  
}  

int sha256mac_encode(const char * key, unsigned int key_length,  
                const char * input, unsigned int input_length,  
                unsigned char * output, unsigned int &output_length) {  
    const EVP_MD * engine = EVP_sha256();  
    HMAC_CTX ctx;  
    HMAC_CTX_init(&ctx);  
    HMAC_Init_ex(&ctx, key, key_length, engine, NULL);  
    HMAC_Update(&ctx, (unsigned char*)input, input_length);        // input is OK; &input is WRONG !!!  
  
    HMAC_Final(&ctx, output, &output_length);  
    HMAC_CTX_cleanup(&ctx);  
  
    return 0;  
}  

int sha256mac(const char* key, int key_len, const char* sign, int sign_len ,unsigned char* out, unsigned int &out_len){	

	sha256mac_encode(key, key_len, sign, sign_len, out, out_len);

    if(out_len != 32){
        std::cout << "out_length: " << out_len << endl;
    }
}

static const std::string base64_chars =   
             "ABCDEFGHIJKLMNOPQRSTUVWXYZ"  
             "abcdefghijklmnopqrstuvwxyz"  
             "0123456789+/";  
  
  
static inline bool is_base64(unsigned char c) {  
  return (isalnum(c) || (c == '+') || (c == '/'));  
}  
  
string base64_encode(unsigned char const* bytes_to_encode, unsigned int in_len) {  
  string ret;  
  int i = 0;  
  int j = 0;  
  unsigned char char_array_3[3];  
  unsigned char char_array_4[4];  
  
  while (in_len--) {  
    char_array_3[i++] = *(bytes_to_encode++);  
    if (i == 3) {  
      char_array_4[0] = (char_array_3[0] & 0xfc) >> 2;  
      char_array_4[1] = ((char_array_3[0] & 0x03) << 4) + ((char_array_3[1] & 0xf0) >> 4);  
      char_array_4[2] = ((char_array_3[1] & 0x0f) << 2) + ((char_array_3[2] & 0xc0) >> 6);  
      char_array_4[3] = char_array_3[2] & 0x3f;  
  
      for(i = 0; (i <4) ; i++)  
        ret += base64_chars[char_array_4[i]];  
      i = 0;  
    }  
  }  
  
  if (i)  
  {  
    for(j = i; j < 3; j++)  
      char_array_3[j] = '\0';  
  
    char_array_4[0] = (char_array_3[0] & 0xfc) >> 2;  
    char_array_4[1] = ((char_array_3[0] & 0x03) << 4) + ((char_array_3[1] & 0xf0) >> 4);  
    char_array_4[2] = ((char_array_3[1] & 0x0f) << 2) + ((char_array_3[2] & 0xc0) >> 6);  
    char_array_4[3] = char_array_3[2] & 0x3f;  
  
    for (j = 0; (j < i + 1); j++)  
      ret += base64_chars[char_array_4[j]];  
  
    while((i++ < 3))  
      ret += '=';  
  
  }  
  
  return ret;  
  
}  

int base64_decode(const std::string& base64_str, unsigned char *bindata)
{
    int i, j;
    unsigned char k;
    unsigned char temp[4];
    const char *base64 = base64_str.c_str();
    for ( i = 0, j = 0; base64[i] != '\0' ; i += 4 )
    {
        memset( temp, 0xFF, sizeof(temp) );
        for ( k = 0 ; k < 64 ; k ++ )
        {
            if ( base64_chars[k] == base64[i] )
                temp[0]= k;
        }
        for ( k = 0 ; k < 64 ; k ++ )
        {
            if ( base64_chars[k] == base64[i+1] )
                temp[1]= k;
        }
        for ( k = 0 ; k < 64 ; k ++ )
        {
            if ( base64_chars[k] == base64[i+2] )
                temp[2]= k;
        }
        for ( k = 0 ; k < 64 ; k ++ )
        {
            if ( base64_chars[k] == base64[i+3] )
                temp[3]= k;
        }

        bindata[j++] = ((unsigned char)(((unsigned char)(temp[0] << 2))&0xFC)) |
                ((unsigned char)((unsigned char)(temp[1]>>4)&0x03));
        if ( base64[i+2] == '=' )
            break;

        bindata[j++] = ((unsigned char)(((unsigned char)(temp[1] << 4))&0xF0)) |
                ((unsigned char)((unsigned char)(temp[2]>>2)&0x0F));
        if ( base64[i+3] == '=' )
            break;

        bindata[j++] = ((unsigned char)(((unsigned char)(temp[2] << 6))&0xF0)) |
                ((unsigned char)(temp[3]&0x3F));
    }
    return j;
}

static inline
unsigned char ToHex(unsigned char x)
{
    static char table[16] = {
        '0', '1', '2', '3', '4', '5', '6', '7',
        '8', '9', 'A', 'B', 'C', 'D', 'E', 'F'
    };
    return table[x];
}

string urlEncode(const string& str)
{
    std::string strTemp = "";
    size_t length = str.length();
    for (size_t i = 0; i < length; i++)
    {
        if (isalnum((unsigned char)str[i]) ||
            (str[i] == '-') ||
            (str[i] == '_') ||
            (str[i] == '.') ||
            (str[i] == '~'))
            strTemp += str[i];
        else
        {
            strTemp += '%';
            strTemp += ToHex((unsigned char)str[i] >> 4);
            strTemp += ToHex((unsigned char)str[i] % 16);
        }
    }
    return strTemp;
}




int aes_encrypt(const string &input_str, const char* key_str, string &output_str)
{
    if (input_str =="" || key_str == NULL)
    {
        return -1;
    }
    AES_KEY aes_key;
    unsigned char ivec[AES_BLOCK_SIZE+1] = "0123234556787890";

    int length = input_str.length();
    int block_num = length/AES_BLOCK_SIZE ;

    char* p_data = (char *)malloc(((block_num+1)*AES_BLOCK_SIZE + 1) * sizeof(char));
    if (p_data == NULL)
    {
        return -3;
    }
    memset(p_data, 0x00,(block_num+1)*AES_BLOCK_SIZE + 1);
    strcpy(p_data, input_str.c_str());
    int k = length % AES_BLOCK_SIZE;
    int j = length / AES_BLOCK_SIZE;
    int padding = AES_BLOCK_SIZE-k;

    for(int i=0;i < padding ; i++)
    {
        p_data[j*AES_BLOCK_SIZE + k + i] = padding;
    }
    p_data[j*AES_BLOCK_SIZE + k + padding] = '\0';

    if(AES_set_encrypt_key((unsigned char*)key_str, 128, &aes_key) < 0)
    {
        return -2;
    }
    char *out = (char *)malloc( ((block_num +1)*AES_BLOCK_SIZE*2 + 1)*sizeof(char));
    memset(out, 0x00, (block_num +1)*AES_BLOCK_SIZE*2+1);
    if (out == NULL)
    {
        SAFE_FREE(out);
        return -3;
    }
    AES_cbc_encrypt((unsigned char*)p_data, (unsigned char*)out, (block_num+1)*AES_BLOCK_SIZE , &aes_key, ivec, AES_ENCRYPT);

    output_str = base64_encode((unsigned char*)out, (block_num + 1)*AES_BLOCK_SIZE);

    SAFE_FREE(out);
    SAFE_FREE(p_data);
    return 0;
}




int aes_decrypt(const string &input_str, const char* key_str, string &output_str)
{
    if (input_str == "" || key_str == NULL)
    {
        return -1;
    }
    AES_KEY aes_key;
    unsigned char ivec[AES_BLOCK_SIZE+1] = "0123234556787890";

    int length = input_str.length();

    char *in_str = (char *)malloc(length * sizeof(char));
    if (in_str == NULL)
    {
        SAFE_FREE(in_str);
        return -3;
    }
    memset(in_str, 0 ,length);
    int tmp_ret = base64_decode(input_str, (unsigned char*)in_str);

    if(AES_set_decrypt_key((unsigned char*)key_str, 128, &aes_key) < 0)
    {
        return -2;
    }
    char *out = (char *)malloc((tmp_ret + 1)*sizeof(char));
    if (out == NULL)
    {
        SAFE_FREE(out);
        return -3;
    }
    memset(out, 0 ,tmp_ret+1);
    AES_cbc_encrypt((unsigned char*)in_str, (unsigned char*)out, tmp_ret, &aes_key, ivec, AES_DECRYPT);

    output_str = string(out,tmp_ret);
    int last_num = out[tmp_ret - 1];

    output_str = output_str.substr(0,output_str.size() - last_num);

    SAFE_FREE(out);
    SAFE_FREE(in_str);
    return 0;
}

