#ifndef _TCLOUD_RECORD_ASR_H_
#define _TCLOUD_RECORD_ASR_H_
#include <string>
#include <map>

using namespace std;

//请求参数结构体
typedef struct tag_TCloudRecordASRReq
{
	string 	EngineModelType;
	int	  	ChannelNum;
	int		ResTextFormat;
	int 	SourceType;
	string  CallbackUrl;
	string 	Url;
	string	Data;
	int 	DataLen;
}TCloudRecordASRReq;


//用户信息结构体
typedef struct tag_TCloudRecordASRAuth
{
	int AppId;
	string SecretId;
	string SecretKey;
}TCloudRecordASRAuth;


class TCloudRecordASR
{
public:
	TCloudRecordASR();
	virtual ~TCloudRecordASR();

public:
	/* 初始化公共请求参数，此类参数较稳定不变
	** configPath 参数文件路径
	** time 鉴权的有效时间
	** Output int 返回结果
	*/
	int InitCommonParam(string configPath);

	/* 初始化用户信息
	** stAuth 用户信息结构体
	** time 鉴权的有效时间
	** Output int 返回结果
	*/
	int InitAuth(TCloudRecordASRAuth stAuth);

	/* 初始化用户信息
	** configPath 参数文件路径
	** time 鉴权的有效时间
	** Output int 返回结果
	*/
	int InitAuth(string configPath);

	/* 创建请求
	** stReq 完整的请求结构体信息
	** Output int 返回结果
	*/
    int CreateRequest(TCloudRecordASRReq stReq);


	/* 请求转换成json
	** stReq 请求结构体
	** Output string json结果
	*/
	string RequestToJson(TCloudRecordASRReq stReq);

	/* 创建识别请求
	** fileURI 音频的本地地址或者音频的URL链接 
	** Output int 返回结果
	*/
	int CreateRecTask(string fileURI);

	/* 轮询查询识别结果
	** taskId  任务Id
	** rsp   查询结果
	** Output int 返回结果
	*/
	int DescribeTaskStatus(int taskId, string &rsp);

	void SetEngineModelType(string  type){
		m_stReq.EngineModelType = type;
	}

	void SetChannelNum(int  num){
		m_stReq.ChannelNum = num;
	}

	void SetResTextFormat(int type){
		m_stReq.ResTextFormat = type;
	}

	/* 设置SourceType
	** type 0 音频url， 1本地音频
	** Output void
	*/
	void SetSourceType(int type){
		m_stReq.SourceType = type;
	}

	void SetCallbackUrl(string url){
		m_stReq.CallbackUrl = url;
	}

	void SetUrl(string url){
		m_stReq.Url = url;
	}

	/* 获取任务结果
	** Output string 任务返回json包
	*/
	string GetResponse(){
		return m_strRsp;
	}
protected:
	//请求参数
	TCloudRecordASRReq  m_stReq;
	//用户信息
	TCloudRecordASRAuth m_stAuth;
	//语音返回结果
    string m_strRsp;
};

#endif
