#include <time.h>
#include <string>
#include <iostream>
#include "TConfigParse.h"
#include "TCloudUtil.h"
#include "TCloudRecordASR.h"
#include "json.h"
#include "encrypt.h"

using namespace std;

#define random(x) (rand()%x)

TCloudRecordASR::TCloudRecordASR(){}

TCloudRecordASR::~TCloudRecordASR(){}

int TCloudRecordASR::InitAuth(TCloudRecordASRAuth stAuth) {
    m_stAuth.AppId     = stAuth.AppId;
    m_stAuth.SecretId  = stAuth.SecretId;
    m_stAuth.SecretKey = stAuth.SecretKey;
}

int TCloudRecordASR::InitAuth(string configPath) {
    CServerConf auth_config;
    int ret = auth_config.ParseFile(configPath.c_str());
    m_stAuth.AppId     = atoi(auth_config["AppId"].c_str());
    m_stAuth.SecretId  = auth_config["SecretId"];
    m_stAuth.SecretKey = auth_config["SecretKey"];
}

int TCloudRecordASR::InitCommonParam(string configPath){
    CServerConf param_config;
    int ret = param_config.ParseFile(configPath.c_str());
   
    m_stReq.EngineModelType = param_config["EngineModelType"];
    m_stReq.ChannelNum      = atoi(param_config["ChannelNum"].c_str());
    m_stReq.ResTextFormat   = atoi(param_config["ResTextFormat"].c_str());
    m_stReq.SourceType      = atoi(param_config["SourceType"].c_str());
    m_stReq.CallbackUrl     = param_config["CallbackUrl"];
}

int TCloudRecordASR::CreateRequest(TCloudRecordASRReq stReq) {
    m_stReq.EngineModelType = stReq.EngineModelType;
    m_stReq.ChannelNum      = stReq.ChannelNum;
    m_stReq.ResTextFormat   = stReq.ResTextFormat;
    m_stReq.SourceType      = stReq.SourceType;
    m_stReq.CallbackUrl     = stReq.CallbackUrl;
}

int TCloudRecordASR::CreateRecTask(std::string fileURI){
    if(fileURI.length()==0||fileURI.compare("")==0){
		cout<<"fileURI cannot empty!";
		return -1;
	}
	int timenow=time(NULL);

	Json::Value jsRoot;
	jsRoot["EngineModelType"]   = m_stReq.EngineModelType;
    jsRoot["ChannelNum"]        = m_stReq.ChannelNum;
    jsRoot["ResTextFormat"]     = m_stReq.ResTextFormat;
    jsRoot["SourceType"]        = m_stReq.SourceType;
    jsRoot["CallbackUrl"]       = m_stReq.CallbackUrl;
    
	if(m_stReq.SourceType == 0){
		//string voice=urlEncode(fileURI);
		jsRoot["Url"] = fileURI;
	}else{
		const char *fwave =fileURI.c_str();
		FILE *fp =fopen(fwave,"rb");
		if(!fp){
			cout<<"the file cannot open!"<<endl;
			return -1;
			}
		else{
			//从本地文件读取音频
			fseek(fp,0,SEEK_END);
			int data_len=ftell(fp);
			jsRoot["DataLen"] =  data_len;
			fseek(fp,0,SEEK_SET);
			char *voicedata=(char*)malloc(sizeof(char)*data_len);
			fread(voicedata,sizeof(char),data_len,fp);
			string data(voicedata, data_len);
			string base64data=base64_encode((unsigned char *)data.c_str(),(unsigned int)data.length());
			jsRoot["Data"] = base64data;
			delete voicedata;
			if(fp!=NULL){
				fclose(fp);
				fp=NULL;
			}
		}

	}
	
	//参数转换成json
	string queryJson = jsRoot.toStyledString();
	string authorization = CalSignature(queryJson, m_stAuth.SecretKey, m_stAuth.SecretId, timenow);
	http_curl_exec(authorization, queryJson, to_string(timenow), m_strRsp, "CreateRecTask");

	return 0;
}

int TCloudRecordASR::DescribeTaskStatus(int taskId, string &rsp){
	int timenow=time(NULL);

	Json::Value jsRoot;
	jsRoot["TaskId"]   = taskId;
    
	//参数转换成json
	string queryJson = jsRoot.toStyledString();
	string authorization = CalSignature(queryJson, m_stAuth.SecretKey, m_stAuth.SecretId, timenow);
	http_curl_exec(authorization, queryJson, to_string(timenow), rsp, "DescribeTaskStatus");

	return 0;
}


