#include "curl/curl.h"
#include <iostream>
#include "TCloudUtil.h"
#include "time.h"
#include "json.h"
#include "encrypt.h"

using namespace std;

string GetToday(){
    time_t t = time(nullptr);
    struct tm *tm = localtime(&t);
    string month = std::to_string(tm->tm_mon+1);
    string day = std::to_string(tm->tm_mday);
    //月份两位数
    if(tm->tm_mon + 1 < 10){
        month = "0" + std::to_string(tm->tm_mon+1);
    }
    //日期两位数
    if(tm->tm_mday < 10){
        day = "0" + std::to_string(tm->tm_mday);
    }
    string today = std::to_string(1900+tm->tm_year) + "-" + month + "-" + day;
    return today;
}

//计算签名
string CalSignature(string queryJson, string secretKey, string secretId, int timenow){

	string sha256_str = sha256(queryJson);
	str2low(sha256_str);
	string canonicalRequest = "POST\n/\n\ncontent-type:application/json\nhost:asr.tencentcloudapi.com\n\ncontent-type;host\n" + sha256_str;
	string canonicalSign = sha256(canonicalRequest);
	
	//签名
	string day = GetToday();
	string credentialScope = day + "/asr/tc3_request";
	string signStr = "TC3-HMAC-SHA256\n" + std::to_string(timenow) + "\n" + credentialScope + "\n" + canonicalSign;
	unsigned char day_sign_rsp[50];
	unsigned int day_rsp_len = 50;
	sha256mac(("TC3" + secretKey).data(), ("TC3" + secretKey).size(), day.data(), day.size(), day_sign_rsp, day_rsp_len);
	unsigned char service_sign_rsp[50];
	unsigned int service_rsp_len = 50;
	sha256mac((const char*)day_sign_rsp, day_rsp_len, "asr", 3, service_sign_rsp, service_rsp_len);
	unsigned char secret_sign_rsp[50];
	unsigned int secret_rsp_len = 50;
	sha256mac((const char *)service_sign_rsp, service_rsp_len,  "tc3_request", 11, secret_sign_rsp, secret_rsp_len);
	unsigned char req_sign_rsp[50];
	unsigned int req_rsp_len = 50;
	sha256mac((const char*)secret_sign_rsp,secret_rsp_len, signStr.data(), signStr.size(), req_sign_rsp, req_rsp_len);
	string signature = string2hex((char*)req_sign_rsp, req_rsp_len);
	str2low(signature);
	string authorization = "TC3-HMAC-SHA256 Credential=" + secretId + "/" + credentialScope + ", " + "SignedHeaders=content-type;host" + ", " + "Signature=" + signature;
	return authorization;
}

size_t write_data(void *buffer, size_t size, size_t count, void *stream) {
	string *pstream = static_cast<string *>(stream);
	(*pstream).append((char *) buffer, size * count);
	return size * count;
}

int http_curl_exec(string &auth, string &queryStr, string timestamp, string &rsp_str, string action){
	string url="https://asr.tencentcloudapi.com";
	curl_global_init(CURL_GLOBAL_ALL);
	CURL* curl = curl_easy_init();
	curl_easy_setopt(curl, CURLOPT_POST, 1);
	curl_easy_setopt(curl, CURLOPT_SSL_VERIFYPEER, false);
	curl_easy_setopt(curl,CURLOPT_SSL_VERIFYHOST,false);
    
   	curl_slist *headers =curl_slist_append(NULL,"Host: asr.tencentcloudapi.com");
   	curl_slist_append(headers,"Content-Type: application/json");
	string authorization = "Authorization: " + auth;
	curl_slist_append(headers,authorization.data());
	curl_slist_append(headers,("X-TC-Action: " + action).data());
	curl_slist_append(headers,("X-TC-Timestamp: "+timestamp).data());
	curl_slist_append(headers,("X-TC-Version: 2019-06-14"));
	curl_easy_setopt(curl,CURLOPT_HTTPHEADER,headers);
	curl_easy_setopt(curl, CURLOPT_URL, url.c_str());
	curl_easy_setopt(curl,CURLOPT_POSTFIELDS,queryStr.c_str());
	curl_easy_setopt(curl,CURLOPT_TIMEOUT,100);
	curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, write_data);
	curl_easy_setopt(curl, CURLOPT_WRITEDATA, &rsp_str);
	CURLcode res= curl_easy_perform(curl);
    if(res != CURLE_OK){
    	string mess=curl_easy_strerror(res);
    	cout<<"curl_easy_perform() failed:"<<mess<<endl;;
        curl_easy_cleanup(curl);
        return -1;
    }
	curl_easy_cleanup(curl);
	curl_global_cleanup();
	return 0;
}